@@ -867,14 +867,16 @@ MYBOOL __WINAPI MPS_readex(lprec **newlp, void *userhandle, read_modeldata_func
           if(var < 0) /* undefined var and could add ... */;
           else if(strcmp(field1, "UP") == 0) {
           /* upper bound */
-            if(!set_bounds(lp, var, get_lowbo(lp, var), field4))
+            /* if(!set_bounds(lp, var, get_lowbo(lp, var), field4)) */
+            if(!set_upbo(lp, var, field4))
               break;
           }
           else if(strcmp(field1, "SC") == 0) {
             /* upper bound */
             if(field4 == 0)
               field4 = lp->infinite;
-            if(!set_bounds(lp, var, get_lowbo(lp, var), field4))
+            /* if(!set_bounds(lp, var, get_lowbo(lp, var), field4)) */
+            if(!set_upbo(lp, var, field4))
               break;
             set_semicont(lp, var, TRUE);
           }
@@ -882,22 +884,26 @@ MYBOOL __WINAPI MPS_readex(lprec **newlp, void *userhandle, read_modeldata_func
             /* upper bound */
             if(field4 == 0)
               field4 = lp->infinite;
-            if(!set_bounds(lp, var, get_lowbo(lp, var), field4))
+            /* if(!set_bounds(lp, var, get_lowbo(lp, var), field4)) */
+            if(!set_upbo(lp, var, field4))
               break;
             set_int(lp, var, TRUE);
             set_semicont(lp, var, TRUE);
           }
           else if(strcmp(field1, "LO") == 0) {
             /* lower bound */
-            if(!set_bounds(lp, var, field4, get_upbo(lp, var)))
+            /* if(!set_bounds(lp, var, field4, get_upbo(lp, var))) */
+            if(!set_lowbo(lp, var, field4))
               break;
           }
-      else if(strcmp(field1, "PL") == 0) { /* plus-ranged variable */
-            if(!set_bounds(lp, var, get_lowbo(lp, var), lp->infinite))
+          else if(strcmp(field1, "PL") == 0) { /* plus-ranged variable */
+            /* if(!set_bounds(lp, var, get_lowbo(lp, var), lp->infinite)) */
+            if(!set_upbo(lp, var, lp->infinite))
               break;
-      }
+          }
           else if(strcmp(field1, "MI") == 0) { /* minus-ranged variable */
-            if(!set_bounds(lp, var, -lp->infinite, get_upbo(lp, var)))
+            /* if(!set_bounds(lp, var, -lp->infinite, get_upbo(lp, var))) */
+            if(!set_lowbo(lp, var, -lp->infinite))
               break;
           }
           else if(strcmp(field1, "FR") == 0) { /* free variable */
@@ -913,12 +919,14 @@ MYBOOL __WINAPI MPS_readex(lprec **newlp, void *userhandle, read_modeldata_func
           }
           /* AMPL bounds type UI and LI added by E.Imamura (CRIEPI)  */
           else if(strcmp(field1, "UI") == 0) { /* upper bound for integer variable */
-            if(!set_bounds(lp, var, get_lowbo(lp, var), field4))
+            /* if(!set_bounds(lp, var, get_lowbo(lp, var), field4)) */
+            if(!set_upbo(lp, var, field4))
               break;
             set_int(lp, var, TRUE);
           }
           else if(strcmp(field1, "LI") == 0) { /* lower bound for integer variable - corrected by KE */
-            if(!set_bounds(lp, var, field4, get_upbo(lp, var)))
+            /* if(!set_bounds(lp, var, field4, get_upbo(lp, var))) */
+            if(!set_lowbo(lp, var, field4))
               break;
             set_int(lp, var, TRUE);
           }
@@ -1230,9 +1238,7 @@ static void number(char *str,REAL value)
   strncpy(str,_str,12);
  }
 
-static char numberbuffer[15];
-
-static char *formatnumber12(double a)
+static char *formatnumber12(char *numberbuffer, double a)
 {
 #if 0
   return(sprintf(numberbuffer, "%12g", a));
@@ -1242,18 +1248,16 @@ static char *formatnumber12(double a)
 #endif
 }
 
-STATIC char *MPSnameFIXED(char *name)
+STATIC char *MPSnameFIXED(char *name0, char *name)
 {
-  static char name0[9];
-
   sprintf(name0, "%-8.8s", name);
   return(name0);
 }
 
-STATIC char *MPSnameFREE(char *name)
+STATIC char *MPSnameFREE(char *name0, char *name)
 {
   if(strlen(name) < 8)
-    return(MPSnameFIXED(name));
+    return(MPSnameFIXED(name0, name));
   else
     return(name);
 }
@@ -1269,12 +1273,14 @@ static void write_data(void *userhandle, write_modeldata_func write_modeldata, c
   write_modeldata(userhandle, buff);
 }
 
-MYBOOL MPS_writefileex(lprec *lp, int typeMPS, void *userhandle, write_modeldata_func write_modeldata)
+MYBOOL __WINAPI MPS_writefileex(lprec *lp, int typeMPS, void *userhandle, write_modeldata_func write_modeldata)
 {
   int    i, j, jj, je, k, marker, putheader, ChangeSignObj = FALSE, *idx, *idx1;
   MYBOOL ok = TRUE, names_used;
   REAL   a, *val, *val1;
-  char * (*MPSname)(char *name);
+  char * (*MPSname)(char *name0, char *name);
+  char numberbuffer[15];
+  char name0[9];
 
   if(lp->matA->is_roworder) {
     report(lp, IMPORTANT, "MPS_writefile: Cannot write to MPS file while in row entry mode.\n");
@@ -1311,6 +1317,9 @@ MYBOOL MPS_writefileex(lprec *lp, int typeMPS, void *userhandle, write_modeldata
     lp->names_used = FALSE;
     ok = TRUE;
   }
+
+  memset(numberbuffer, 0, sizeof(numberbuffer));
+
   marker = 0;
 
   /* First write metadata in structured comment form (lp_solve style) */
@@ -1328,7 +1337,7 @@ MYBOOL MPS_writefileex(lprec *lp, int typeMPS, void *userhandle, write_modeldata
   write_data(userhandle, write_modeldata, "*\n");
 
   /* Write the MPS content */
-  write_data(userhandle, write_modeldata, "NAME          %s\n", MPSname(get_lp_name(lp)));
+  write_data(userhandle, write_modeldata, "NAME          %s\n", MPSname(name0, get_lp_name(lp)));
   if((typeMPS == MPSFREE) && (is_maxim(lp)))
     write_data(userhandle, write_modeldata, "OBJSENSE\n MAX\n");
   write_data(userhandle, write_modeldata, "ROWS\n");
@@ -1343,7 +1352,7 @@ MYBOOL MPS_writefileex(lprec *lp, int typeMPS, void *userhandle, write_modeldata
     }
     else
       write_data(userhandle, write_modeldata, " E  ");
-    write_data(userhandle, write_modeldata, "%s\n", MPSname(get_row_name(lp, i)));
+    write_data(userhandle, write_modeldata, "%s\n", MPSname(name0, get_row_name(lp, i)));
   }
 
   allocREAL(lp, &val, 1 + lp->rows, TRUE);
@@ -1370,17 +1379,17 @@ MYBOOL MPS_writefileex(lprec *lp, int typeMPS, void *userhandle, write_modeldata
         a = *(val1++);
         if (k == 0) {
           write_data(userhandle, write_modeldata, "    %s",
-                          MPSname(get_col_name(lp, i)));
+                          MPSname(name0, get_col_name(lp, i)));
           write_data(userhandle, write_modeldata, "  %s  %s",
-                          MPSname(get_row_name(lp, j)),
-/*                          formatnumber12((double) a)); */
-                          formatnumber12((double) (a * (j == 0 && ChangeSignObj ? -1 : 1))));
+                          MPSname(name0, get_row_name(lp, j)),
+/*                          formatnumber12(numberbuffer, (double) a)); */
+                          formatnumber12(numberbuffer, (double) (a * (j == 0 && ChangeSignObj ? -1 : 1))));
     }
         else
           write_data(userhandle, write_modeldata, "   %s  %s\n",
-                          MPSname(get_row_name(lp, j)),
-                          formatnumber12((double) (a * (j == 0 && ChangeSignObj ? -1 : 1))));
-/*                          formatnumber12((double) a)); */
+                          MPSname(name0, get_row_name(lp, j)),
+                          formatnumber12(numberbuffer, (double) (a * (j == 0 && ChangeSignObj ? -1 : 1))));
+/*                          formatnumber12(numberbuffer, (double) a)); */
       }
       if(k == 0)
         write_data(userhandle, write_modeldata, "\n");
@@ -1404,12 +1413,12 @@ MYBOOL MPS_writefileex(lprec *lp, int typeMPS, void *userhandle, write_modeldata
       k = 1 - k;
       if(k == 0)
         write_data(userhandle, write_modeldata, "    RHS       %s  %s",
-                        MPSname(get_row_name(lp, i)),
-                        formatnumber12((double)a));
+                        MPSname(name0, get_row_name(lp, i)),
+                        formatnumber12(numberbuffer, (double)a));
       else
         write_data(userhandle, write_modeldata, "   %s  %s\n",
-                        MPSname(get_row_name(lp, i)),
-                        formatnumber12((double)a));
+                        MPSname(name0, get_row_name(lp, i)),
+                        formatnumber12(numberbuffer, (double)a));
     }
   }
   if(k == 0)
@@ -1429,12 +1438,12 @@ MYBOOL MPS_writefileex(lprec *lp, int typeMPS, void *userhandle, write_modeldata
       k = 1 - k;
       if(k == 0)
         write_data(userhandle, write_modeldata, "    RGS       %s  %s",
-                        MPSname(get_row_name(lp, i)),
-                        formatnumber12((double)a));
+                        MPSname(name0, get_row_name(lp, i)),
+                        formatnumber12(numberbuffer, (double)a));
       else
         write_data(userhandle, write_modeldata, "   %s  %s\n",
-                        MPSname(get_row_name(lp, i)),
-                        formatnumber12((double)a));
+                        MPSname(name0, get_row_name(lp, i)),
+                        formatnumber12(numberbuffer, (double)a));
     }
   }
   if(k == 0)
@@ -1453,8 +1462,8 @@ MYBOOL MPS_writefileex(lprec *lp, int typeMPS, void *userhandle, write_modeldata
           putheader = FALSE;
         }
         write_data(userhandle, write_modeldata, " FX BND       %s  %s\n",
-                        MPSname(get_col_name(lp, j)),
-                        formatnumber12((double)a));
+                        MPSname(name0, get_col_name(lp, j)),
+                        formatnumber12(numberbuffer, (double)a));
       }
       else if(is_binary(lp, j)) {
         if(putheader) {
@@ -1462,7 +1471,7 @@ MYBOOL MPS_writefileex(lprec *lp, int typeMPS, void *userhandle, write_modeldata
           putheader = FALSE;
         }
         write_data(userhandle, write_modeldata, " BV BND       %s\n",
-                        MPSname(get_col_name(lp, j)));
+                        MPSname(name0, get_col_name(lp, j)));
       }
       else if(is_unbounded(lp, j)) {
         if(putheader) {
@@ -1470,33 +1479,10 @@ MYBOOL MPS_writefileex(lprec *lp, int typeMPS, void *userhandle, write_modeldata
           putheader = FALSE;
         }
         write_data(userhandle, write_modeldata, " FR BND       %s\n",
-                        MPSname(get_col_name(lp, j)));
+                        MPSname(name0, get_col_name(lp, j)));
       }
       else {
-        if((lp->orig_upbo[i] < lp->infinite) || (is_semicont(lp, j))) {
-          a = lp->orig_upbo[i];
-      if(a < lp->infinite)
-            a = unscaled_value(lp, a, i);
-          if(putheader) {
-            write_data(userhandle, write_modeldata, "BOUNDS\n");
-            putheader = FALSE;
-          }
-          if(is_semicont(lp, j)) {
-            if(is_int(lp, j))
-              write_data(userhandle, write_modeldata, " SI BND       %s  %s\n",
-                              MPSname(get_col_name(lp, j)),
-                  (a < lp->infinite) ? formatnumber12((double)a) : "            ");
-            else
-              write_data(userhandle, write_modeldata, " SC BND       %s  %s\n",
-                              MPSname(get_col_name(lp, j)),
-                              (a < lp->infinite) ? formatnumber12((double)a) : "            ");
-          }
-          else
-            write_data(userhandle, write_modeldata, " UP BND       %s  %s\n",
-                            MPSname(get_col_name(lp, j)),
-                            formatnumber12((double)a));
-        }
-        if(lp->orig_lowbo[i] != 0) {
+        if((lp->orig_lowbo[i] != 0) || (is_int(lp, j))) { /* Some solvers like CPLEX need to have a bound on a variable if it is integer, but not binary else it is interpreted as binary which is not ment */
           a = lp->orig_lowbo[i];
           a = unscaled_value(lp, a, i);
           if(putheader) {
@@ -1505,11 +1491,35 @@ MYBOOL MPS_writefileex(lprec *lp, int typeMPS, void *userhandle, write_modeldata
           }
           if(lp->orig_lowbo[i] != -lp->infinite)
             write_data(userhandle, write_modeldata, " LO BND       %s  %s\n",
-                            MPSname(get_col_name(lp, j)),
-                            formatnumber12((double)a));
+                            MPSname(name0, get_col_name(lp, j)),
+                            formatnumber12(numberbuffer, (double)a));
           else
             write_data(userhandle, write_modeldata, " MI BND       %s\n",
-                            MPSname(get_col_name(lp, j)));
+                            MPSname(name0, get_col_name(lp, j)));
+        }
+
+        if((lp->orig_upbo[i] < lp->infinite) || (is_semicont(lp, j))) {
+          a = lp->orig_upbo[i];
+          if(a < lp->infinite)
+            a = unscaled_value(lp, a, i);
+          if(putheader) {
+            write_data(userhandle, write_modeldata, "BOUNDS\n");
+            putheader = FALSE;
+          }
+          if(is_semicont(lp, j)) {
+            if(is_int(lp, j))
+              write_data(userhandle, write_modeldata, " SI BND       %s  %s\n",
+                              MPSname(name0, get_col_name(lp, j)),
+                  (a < lp->infinite) ? formatnumber12(numberbuffer, (double)a) : "            ");
+            else
+              write_data(userhandle, write_modeldata, " SC BND       %s  %s\n",
+                              MPSname(name0, get_col_name(lp, j)),
+                              (a < lp->infinite) ? formatnumber12(numberbuffer, (double)a) : "            ");
+          }
+          else
+            write_data(userhandle, write_modeldata, " UP BND       %s  %s\n",
+                            MPSname(name0, get_col_name(lp, j)),
+                            formatnumber12(numberbuffer, (double)a));
         }
       }
     }
@@ -1525,12 +1535,12 @@ MYBOOL MPS_writefileex(lprec *lp, int typeMPS, void *userhandle, write_modeldata
     }
     write_data(userhandle, write_modeldata, " S%1d SOS       %s  %s\n",
                     SOS->sos_list[i]->type,
-                    MPSname(SOS->sos_list[i]->name),
-                    formatnumber12((double) SOS->sos_list[i]->priority));
+                    MPSname(name0, SOS->sos_list[i]->name),
+                    formatnumber12(numberbuffer, (double) SOS->sos_list[i]->priority));
     for(j = 1; j <= SOS->sos_list[i]->size; j++) {
       write_data(userhandle, write_modeldata, "    SOS       %s  %s\n",
-                      MPSname(get_col_name(lp, SOS->sos_list[i]->members[j])),
-                      formatnumber12((double) SOS->sos_list[i]->weights[j]));
+                      MPSname(name0, get_col_name(lp, SOS->sos_list[i]->members[j])),
+                      formatnumber12(numberbuffer, (double) SOS->sos_list[i]->weights[j]));
     }
   }
 
@@ -1764,7 +1774,8 @@ MYBOOL MPS_writeBAS(lprec *lp, int typeMPS, char *filename)
   MYBOOL ok;
   char   name1[100], name2[100];
   FILE   *output = stdout;
-  char * (*MPSname)(char *name);
+  char * (*MPSname)(char *name0, char *name);
+  char name0[9];
 
   /* Set name formatter */
   switch(typeMPS) {
@@ -1806,16 +1817,16 @@ MYBOOL MPS_writeBAS(lprec *lp, int typeMPS, char *filename)
 
     /* Check if we have a basic/non-basic variable pair */
     if((ib <= lp->sum) && (in <= lp->sum)) {
-      strcpy(name1, MPSname((ib <= lp->rows ? get_row_name(lp, ib) :
+      strcpy(name1, MPSname(name0, (ib <= lp->rows ? get_row_name(lp, ib) :
                                               get_col_name(lp, ib-lp->rows))));
-      strcpy(name2, MPSname((in <= lp->rows ? get_row_name(lp, in) :
+      strcpy(name2, MPSname(name0, (in <= lp->rows ? get_row_name(lp, in) :
                                               get_col_name(lp, in-lp->rows))));
       fprintf(output, " %2s %s  %s\n", (lp->is_lower[in] ? "XL" : "XU"), name1, name2);
     }
 
     /* Otherwise just write the bound state of the non-basic variable */
     else if(in <= lp->sum) {
-      strcpy(name1, MPSname((in <= lp->rows ? get_row_name(lp, in) :
+      strcpy(name1, MPSname(name0, (in <= lp->rows ? get_row_name(lp, in) :
                                               get_col_name(lp, in-lp->rows))));
       fprintf(output, " %2s %s\n", (lp->is_lower[in] ? "LL" : "UL"), name1);
     }
