@@ -1,6 +1,7 @@
 #include <string.h>
 #include <time.h>
 #include <signal.h>
+#include <math.h>
 #include "lp_lib.h"
 
 #ifdef FORTIFY
@@ -40,13 +41,15 @@ void print_help(char *argv[])
 #endif
   printf("-mps\t\tread from MPS file in fixed format\n");
   printf("-fmps\t\tread from MPS file in free format\n");
+  printf("-rpar filename\tread parameters from filename.\n");
+  printf("-rparopt options\n\t\toptions for parameter file:\n");
+  printf("\t\t -H headername: header name for parameters. By default 'Default'\n");
   printf("-rxli xliname filename\n\t\tread file with xli library\n");
   printf("-rxlidata datafilename\n\t\tdata file name for xli library.\n");
   printf("-rxliopt options\n\t\toptions for xli library.\n");
   printf("-rbas filename\tread basis from filename.\n");
-  printf("-rpar filename\tread parameters from filename.\n");
-  printf("-rparopt options\n\t\toptions for parameter file:\n");
-  printf("\t\t -H headername: header name for parameters. By default 'Default'\n");
+  printf("-gbas filename\tguess basis with variables from filename.\n");
+  printf("-plp\t\tprint model.\n");
   printf("-wlp filename\twrite to LP file\n");
   printf("-wmps filename\twrite to MPS file in fixed format\n");
   printf("-wfmps filename\twrite to MPS file in free format\n");
@@ -215,6 +218,7 @@ void print_help(char *argv[])
   printf("-Da <filename>\tDo a generic readable data dump of key lp_solve model variables\n\t\tafter solve.\n\t\tPrincipally for run difference and debugging purposes\n");
   printf("-i\t\tprint all intermediate valid solutions.\n\t\tCan give you useful solutions even if the total run time\n\t\tis too long\n");
   printf("-ia\t\tprint all intermediate (only non-zero values) valid solutions.\n\t\tCan give you useful solutions even if the total run time\n\t\tis too long\n");
+  printf("-stat\t\tPrint model statistics\n");
   printf("-S <detail>\tPrint solution. If detail omitted, then -S2 is used.\n");
   printf("\t -S0: Print nothing\n");
   printf("\t -S1: Only objective value\n");
@@ -272,8 +276,11 @@ static void __WINAPI LPMessageCB(lprec *lp, void *USERHANDLE, int msg)
     DoReport(lp, "Improved MILP ");
 }
 
-void write_model(lprec *lp, char *wlp, char *wmps, char *wfmps, char *wxli, char *wxlisol, char *wxliname, char *wxlioptions)
+void write_model(lprec *lp, char plp, char *wlp, char *wmps, char *wfmps, char *wxli, char *wxlisol, char *wxliname, char *wxlioptions)
 {
+  if(plp)
+    print_lp(lp);
+
   if(wlp != NULL)
     write_lp(lp, wlp);
 
@@ -314,10 +321,208 @@ static void set_value(int *value, int orvalue)
   *value = orvalue;
 }
 
+#define nstats 5
+
+static void minmax1(REAL value0, REAL factor, REAL *minmax, int *nminmax, int row, int *rowminmax, int col, int *colminmax)
+{
+  int i, n;
+  REAL value;
+
+  if (value0 == 0)
+    return;
+  value = fabs(value0) * factor;
+  for (i = 0; (i < *nminmax) && (value <= fabs(minmax[i]) * factor); i++)
+    if (value == fabs(minmax[i]) * factor)
+	  return;
+  if (i >= nstats)
+    return;
+  n = *nminmax;
+  if (n == nstats)
+    n--;
+  memmove(minmax + i + 1, minmax + i, (n - i) * sizeof(*minmax));
+  minmax[i] = value0;
+  if (rowminmax != NULL) {
+    memmove(rowminmax + i + 1, rowminmax + i, (n - i) * sizeof(*rowminmax));
+    rowminmax[i] = row;
+  }
+  if (colminmax != NULL) {
+    memmove(colminmax + i + 1, colminmax + i, (n - i) * sizeof(*colminmax));
+    colminmax[i] = col;
+  }
+  if (*nminmax < nstats)
+    (*nminmax)++;
+}
+
+static void minmax(REAL value, REAL *minima, REAL *maxima, int *nminima, int *nmaxima, int row, int *rowmin, int *rowmax, int col, int *colmin, int *colmax)
+{
+  minmax1(value, -1.0, minima, nminima, row, rowmin, col, colmin);
+  minmax1(value, +1.0, maxima, nmaxima, row, rowmax, col, colmax);
+}
+
+/*
+static void printminmax1(lprec *lp, char *s, REAL *minmax, int nminmax, int *rowminmax, int *colminmax)
+{
+  int i;
+
+  for (i = 0; i < nminmax; i++) {
+    printf(" %s(", s);
+    if (rowminmax != NULL)
+	  printf("%s", get_row_name(lp, rowminmax[i]));
+	if ((rowminmax != NULL) && (colminmax != NULL))
+	  printf(", ");
+	if (colminmax != NULL)
+	  printf("%s", get_col_name(lp, colminmax[i]));
+	printf(") = %.8f\n", minmax[i]);
+  }
+}
+
+static void printminmax(lprec *lp, char *s, REAL *minima, REAL *maxima, int nminima, int nmaxima, int *rowmin, int *rowmax, int *colmin, int *colmax)
+{
+  printf("Minima:\n");
+  printminmax1(lp, s, minima, nminima, rowmin, colmin);
+  printf("Maxima:\n");
+  printminmax1(lp, s, maxima, nmaxima, rowmax, colmax);
+}
+*/
+
+static void printminmax(lprec *lp, char *s, REAL *minima, REAL *maxima, int nminima, int nmaxima, int *rowmin, int *rowmax, int *colmin, int *colmax)
+{
+  int i, nminmax, n;
+
+  nminmax = nminima;
+  if (nmaxima > nminmax)
+    nminmax = nmaxima;
+  for (i = 0; i < nminmax; i++) {
+    n = 0;
+	if (i < nminima) {
+      n += printf("%s(", s);
+      if (rowmin != NULL)
+	    n+= printf("%s", get_row_name(lp, rowmin[i]));
+	  if ((rowmin != NULL) && (colmin != NULL))
+	    n += printf(", ");
+	  if (colmin != NULL)
+	    n += printf("%s", get_col_name(lp, colmin[i]));
+	  n += printf(") = %.8f", minima[i]);
+	}
+	if (n < 40)
+	  n = 40 - n;
+	else
+	  n = 1;
+	printf("%*.*s", n, n, "");
+
+    if (i < nmaxima) {
+      n += printf("%s(", s);
+      if (rowmax != NULL)
+	    n+= printf("%s", get_row_name(lp, rowmax[i]));
+	  if ((rowmax != NULL) && (colmax != NULL))
+	    n += printf(", ");
+	  if (colmax != NULL)
+	    n += printf("%s", get_col_name(lp, colmax[i]));
+	  n += printf(") = %.8f", maxima[i]);
+	}
+
+	printf("\n");
+  }
+}
+
+static void print_statistics(lprec *lp)
+{
+  REAL *col, *RHSmin, *RHSmax, *OBJmin, *OBJmax, *MATmin, *MATmax;
+  int *nz, ret, m, n, i, j, k, l, nRHSmin = 0, nRHSmax = 0, nOBJmin = 0, nOBJmax = 0, nMATmin = 0, nMATmax = 0, *rowRHSmin, *rowRHSmax, *colOBJmin, *colOBJmax, *rowMATmin, *rowMATmax, *colMATmin, *colMATmax;
+
+  m = get_Nrows(lp);
+  n = get_Ncolumns(lp);
+
+  col = (REAL *) malloc((1 + m) * sizeof(*col));
+  nz = (int *) malloc((1 + m) * sizeof(*RHSmin));
+  RHSmin = (REAL *) malloc(nstats * sizeof(*RHSmin));
+  RHSmax = (REAL *) malloc(nstats * sizeof(*RHSmax));
+  rowRHSmin = (int *) malloc(nstats * sizeof(*RHSmin));
+  rowRHSmax = (int *) malloc(nstats * sizeof(*RHSmax));
+  OBJmin = (REAL *) malloc(nstats * sizeof(*OBJmin));
+  OBJmax = (REAL *) malloc(nstats * sizeof(*OBJmax));
+  colOBJmin = (int *) malloc(nstats * sizeof(*colOBJmin));
+  colOBJmax = (int *) malloc(nstats * sizeof(*colOBJmax));
+  MATmin = (REAL *) malloc(nstats * sizeof(*MATmin));
+  MATmax = (REAL *) malloc(nstats * sizeof(*MATmax));
+  rowMATmin = (int *) malloc(nstats * sizeof(*rowMATmin));
+  rowMATmax = (int *) malloc(nstats * sizeof(*rowMATmax));
+  colMATmin = (int *) malloc(nstats * sizeof(*colMATmin));
+  colMATmax = (int *) malloc(nstats * sizeof(*colMATmax));
+
+/*
+  minmax(2.0, MATmin, MATmax, &nMATmin, &nMATmax, 1, rowMATmin, rowMATmax, 8, colMATmin, colMATmax);
+  minmax(1.0, MATmin, MATmax, &nMATmin, &nMATmax, 2, rowMATmin, rowMATmax, 7, colMATmin, colMATmax);
+  minmax(1.5, MATmin, MATmax, &nMATmin, &nMATmax, 3, rowMATmin, rowMATmax, 6, colMATmin, colMATmax);
+  minmax(3.0, MATmin, MATmax, &nMATmin, &nMATmax, 4, rowMATmin, rowMATmax, 5, colMATmin, colMATmax);
+  minmax(4.0, MATmin, MATmax, &nMATmin, &nMATmax, 5, rowMATmin, rowMATmax, 4, colMATmin, colMATmax);
+  minmax(0.1, MATmin, MATmax, &nMATmin, &nMATmax, 6, rowMATmin, rowMATmax, 3, colMATmin, colMATmax);
+  minmax(5.0, MATmin, MATmax, &nMATmin, &nMATmax, 7, rowMATmin, rowMATmax, 2, colMATmin, colMATmax);
+  minmax(1.4, MATmin, MATmax, &nMATmin, &nMATmax, 8, rowMATmin, rowMATmax, 1, colMATmin, colMATmax);
+
+  printminmax(MATmin, MATmax, nMATmin, nMATmax, rowMATmin, rowMATmax, colMATmin, colMATmax);
+*/
+
+  for (i = 1; i <= m; i++)
+    minmax(get_rh(lp, i), RHSmin, RHSmax, &nRHSmin, &nRHSmax, i, rowRHSmin, rowRHSmax, 0, NULL, NULL);
+
+  for (j = 1; j <= n; j++) {
+    ret = get_columnex(lp, j, col, nz);
+	for (i = 0; i < ret; i++)
+	  if (nz[i] == 0)
+        minmax(col[i], OBJmin, OBJmax, &nOBJmin, &nOBJmax, 0, NULL, NULL, j, colOBJmin, colOBJmax);
+	  else
+        minmax(col[i], MATmin, MATmax, &nMATmin, &nMATmax, nz[i], rowMATmin, rowMATmax, j, colMATmin, colMATmax);
+  }
+
+  printf("Constraints: %d\n", get_Nrows(lp));
+  printf("Variables  : %d\n", get_Ncolumns(lp));
+  for (j = k = l = 0, i = n; i >= 1; i--) {
+    if (is_int(lp, i))
+	  j++;
+	if (is_semicont(lp, i))
+	  k++;
+	if (is_SOS_var(lp, i))
+	  l++;
+  }
+  printf("Integers   : %d\n", j);
+  printf("Semi-cont  : %d\n", k);
+  printf("SOS        : %d\n", l);
+  k = get_nonzeros(lp);
+  printf("Non-zeros  : %d\tdensity=%f%%\n", k, ((double) k) / (((double) m) * ((double) n)) * 100.0);
+
+  printf("\nAbsolute Ranges:\n\n       Minima                                  Maxima\n");
+  printf("\nMatrix Coeficients:\n");
+  printminmax(lp, "A", MATmin, MATmax, nMATmin, nMATmax, rowMATmin, rowMATmax, colMATmin, colMATmax);
+
+  printf("\nObj. Vector:\n");
+  printminmax(lp, "c", OBJmin, OBJmax, nOBJmin, nOBJmax, NULL, NULL, colOBJmin, colOBJmax);
+
+  printf("\nRHS Vector:\n");
+  printminmax(lp, "b", RHSmin, RHSmax, nRHSmin, nRHSmax, rowRHSmin, rowRHSmax, NULL, NULL);
+
+  free(col);
+  free(nz);
+  free(RHSmin);
+  free(RHSmax);
+  free(rowRHSmin);
+  free(rowRHSmax);
+  free(OBJmin);
+  free(OBJmax);
+  free(colOBJmin);
+  free(colOBJmax);
+  free(MATmin);
+  free(MATmax);
+  free(rowMATmin);
+  free(rowMATmax);
+  free(colMATmin);
+  free(colMATmax);
+}
+
 int main(int argc, char *argv[])
 {
   lprec *lp = NULL;
-  char *filen, *wlp = NULL, *wmps = NULL, *wfmps = NULL;
+  char *filen, *wlp = NULL, *wmps = NULL, *wfmps = NULL, plp = FALSE;
   int i;
   int verbose = IMPORTANT /* CRITICAL */;
   int debug = -1;
@@ -326,6 +531,7 @@ int main(int argc, char *argv[])
   MYBOOL write_model_after = FALSE;
   MYBOOL noint = FALSE;
   int print_sol = -1;
+  MYBOOL print_stats = FALSE;
   int floor_first = -1;
   MYBOOL do_set_bb_depthlimit = FALSE;
   int bb_depthlimit = 0;
@@ -352,6 +558,7 @@ int main(int argc, char *argv[])
   int scalemode1 = -1;
   int scalemode2 = -1;
   int crashmode = -1;
+  char *guessbasis = NULL;
   /* short timeoutok = FALSE; */
   long sectimeout = -1;
   int result;
@@ -405,6 +612,8 @@ int main(int argc, char *argv[])
       print_sol = TRUE;
     else if(strcmp(argv[i], "-ia") == 0)
       print_sol = AUTOMATIC;
+	else if(strcmp(argv[i], "-stat") == 0)
+	  print_stats = TRUE;
     else if(strcmp(argv[i], "-nonames") == 0)
       nonames = TRUE;
     else if(strcmp(argv[i], "-norownames") == 0)
@@ -543,6 +752,8 @@ int main(int argc, char *argv[])
     }
     else if(strncmp(argv[i], "-C", 2) == 0)
       crashmode = atoi(argv[i] + 2);
+	else if((strcmp(argv[i],"-gbas") == 0) && (i + 1 < argc))
+      guessbasis = argv[++i];
     else if(strcmp(argv[i], "-t") == 0)
       tracing = TRUE;
     else if(strncmp(argv[i], "-S", 2) == 0) {
@@ -591,6 +802,8 @@ int main(int argc, char *argv[])
 #endif
     else if((strcmp(argv[i],"-wlp") == 0) && (i + 1 < argc))
       wlp = argv[++i];
+	else if(strcmp(argv[i],"-plp") == 0)
+      plp = TRUE;
     else if(strcmp(argv[i],"-mps") == 0)
       filetype = filetypeMPS;
     else if(strcmp(argv[i],"-fmps") == 0)
@@ -842,7 +1055,10 @@ int main(int argc, char *argv[])
   }
 
   if(!write_model_after)
-    write_model(lp, wlp, wmps, wfmps, wxli, NULL, wxliname, wxlioptions);
+    write_model(lp, plp, wlp, wmps, wfmps, wxli, NULL, wxliname, wxlioptions);
+
+  if(print_stats)
+    print_statistics(lp);
 
   if(parse_only) {
     if(!write_model_after) {
@@ -954,6 +1170,55 @@ int main(int argc, char *argv[])
     set_scalelimit(lp, scaleloop);
   }
 
+  if(guessbasis != NULL) {
+    REAL *guessvector, a;
+	int *basisvector;
+	int Nrows = get_Nrows(lp);
+	int Ncolumns = get_Ncolumns(lp);
+	int col;
+	char buf[50], *ptr;
+	FILE *fp;
+
+	if ((fp = fopen(guessbasis, "r")) != NULL) {
+	  guessvector = (REAL *) calloc(1+Ncolumns, sizeof(*guessvector));
+	  basisvector = (int *) malloc((1+Nrows+Ncolumns)*sizeof(*basisvector));
+	  if ((guessvector != NULL) && (basisvector != NULL)) {
+	    while ((!feof(fp)) && (fgets(buf, sizeof(buf), fp) != NULL)) {
+  		  ptr = strrchr(buf, ':');
+		  if (ptr == NULL) {
+			printf("Mallformed line: %s\n", buf);
+		  }
+		  else {
+		    a = atof(ptr + 1);
+		    while ((ptr > buf) && (isspace(ptr[-1])))
+			  ptr--;
+		    *ptr = 0;
+			col = get_nameindex(lp, buf, FALSE);
+			if (col < 1)
+			  printf("guess_basis: Unknown variable name %s\n", buf);
+			else
+		      guessvector[col] = a;
+		  }
+		}
+	    if (guess_basis(lp, guessvector, basisvector)) {
+	      if (!set_basis(lp, basisvector, TRUE))
+		    printf("Unable to set guessed basis.\n");
+	    }
+	    else
+	      printf("Unable to guess basis from provided variables.\n");
+	  }
+	  else
+	    printf("guess_basis: Out of memory.\n");
+	  if (basisvector != NULL)
+	    free(basisvector);
+	  if (guessvector != NULL)
+	    free(guessvector);
+	  fclose(fp);
+	}
+	else
+	  printf("Unable to open file %s\n", guessbasis);
+  }
+
   if(rbasname != NULL)
     if(!read_basis(lp, rbasname, NULL)) {
       fprintf(stderr, "Unable to read basis file.\n");
@@ -968,9 +1233,9 @@ int main(int argc, char *argv[])
       fprintf(stderr, "Unable to write basis file.\n");
 
   if(write_model_after)
-    write_model(lp, wlp, wmps, wfmps, wxli, NULL, wxliname, wxlioptions);
+    write_model(lp, plp, wlp, wmps, wfmps, wxli, NULL, wxliname, wxlioptions);
 
-  write_model(lp, NULL, NULL, NULL, NULL, wxlisol, wxliname, wxlisoloptions);
+  write_model(lp, FALSE, NULL, NULL, NULL, NULL, wxlisol, wxliname, wxlisoloptions);
 
   if(PRINT_SOLUTION >= 6)
     print_scales(lp);
